using System;
using System.Runtime.InteropServices;
using System.Text;

namespace ivm {
	public enum Result {
	    ok = 0,
	    error = -1,
	    not_implemented = -2,
	    value_error = -3,
	    no_device = -4
	};
	public static class UrpcBuilderVersion {
	    public const int major = 0;
	    public const int minor = 7;
	    public const int bugfix = 2;
	    public const String suffix = "";
	    public const String version = "0.7.2";
	};
	[Flags] public enum Flags : uint {
		FRAME_SIZE = 25,
		CALIBRATION_OK = 0,
		STATUS_OK = 0,
		MEASUREMENT_NOT_COMPLETE = 0,
		MEASUREMENT_COMPLETE = 1,
		RANKING_NOT_SUPPORTED = 0,
		RANK_PRIMARY = 1,
		MIN_NUMBER_POINTS = 10,
		MAX_NUMBER_POINTS = 1000,
		CURRENT_SENSE_MODE_ISOLATED = 0,
		CURRENT_SENSE_MODE_I_LOW = 1,
		CURRENT_SENSE_MODE_I_MID = 2,
		CURRENT_SENSE_MODE_I_HIGH = 3,
		OUT_MODE_PROBE_SIGNAL_CONTINUOUS = 0,
		OUT_MODE_GROUNDED_CONTINUOUS = 1,
		OUT_MODE_PROBE_SIGNAL_WITH_GROUNDING = 2,
	}
	[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Ansi)]
	public struct in_get_measurement_t {
		public ushort FrameNumber;
	};
	[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Ansi)]
	public struct out_get_measurement_t {
		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 25)]
		public float[] Voltage;
		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 25)]
		public float[] Current;
	};
	[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Ansi)]
	public struct get_identity_information_t {
		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 16)]
		public byte[] Manufacturer;
		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 16)]
		public byte[] ProductName;
		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 16)]
		public byte[] ControllerName;
		public byte HardwareMajor;
		public byte HardwareMinor;
		public ushort HardwareBugfix;
		public byte BootloaderMajor;
		public byte BootloaderMinor;
		public ushort BootloaderBugfix;
		public byte FirmwareMajor;
		public byte FirmwareMinor;
		public ushort FirmwareBugfix;
		public uint SerialNumber;
		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 8)]
		public byte[] Reserved;
	};
	[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Ansi)]
	public struct start_autocalibration_t {
		public byte Result;
	};
	[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Ansi)]
	public struct get_status_t {
		public uint Status;
		public short Temp;
		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 48)]
		public byte[] Reserved;
	};
	[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Ansi)]
	public struct check_measurement_status_t {
		public byte ReadyStatus;
		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 15)]
		public byte[] Reserved;
	};
	[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Ansi)]
	public struct in_get_measurement_raw_t {
		public ushort Frame;
	};
	[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Ansi)]
	public struct out_get_measurement_raw_t {
		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 25)]
		public ushort[] ADCHighCode;
		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 25)]
		public ushort[] ADCLowCode;
	};
	[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Ansi)]
	public struct get_device_rank_t {
		public uint Rank;
		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 128)]
		public byte[] Reserved0;
	};
	[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Ansi)]
	public struct measurement_settings_t {
		public float SamplingRate;
		public float ProbeSignalFrequency;
		public uint NumberPoints;
		public uint NumberChargePoints;
		public float MaxVoltage;
		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 4)]
		public byte[] Reserved0;
		public uint CurrentSensorMode;
		public byte OutputMode;
		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 19)]
		public byte[] Reserved1;
	};
	[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Ansi)]
	public struct calibration_settings_t {
		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 8)]
		public byte[] Reserved0;
		public float ADCMult;
		public float ADCVOffset;
		public float CurrentSense1Mult;
		public float CurrentSense1Offset;
		public float CurrentSense2Mult;
		public float CurrentSense2Offset;
		public float CurrentSense3Mult;
		public float CurrentSense3Offset;
		[MarshalAs(UnmanagedType.ByValArray, SizeConst = 72)]
		public byte[] Reserved1;
	};
	public partial class API {
		[DllImport("ivm", CharSet = CharSet.Ansi, CallingConvention=CallingConvention.Cdecl)]
		public static extern int ivm_open_device([MarshalAs(UnmanagedType.LPStr)] String name);
		            [DllImport("ivm", CharSet = CharSet.Ansi, CallingConvention=CallingConvention.Cdecl)]
		            public static extern int ivm_close_device([MarshalAs(UnmanagedType.LPStr)] String libver);
		        ''').format(
		    libfile=libfile,
		    func_name=namespace_symbol(protocol, "lib_version")
		), "	" * 2))
		out.write(indent(dedent('''            [DllImport("ivm", CallingConvention=CallingConvention.Cdecl)]
		    public static extern Result ivm_close_device(ref int id);
		[DllImport("ivm", CallingConvention=CallingConvention.Cdecl)]
		public static extern Result ivm_get_measurement(int id, ref in_get_measurement_t input, out out_get_measurement_t output);
		[DllImport("ivm", CallingConvention=CallingConvention.Cdecl)]
		public static extern Result ivm_get_identity_information(int id, out get_identity_information_t output);
		[DllImport("ivm", CallingConvention=CallingConvention.Cdecl)]
		public static extern Result ivm_start_autocalibration(int id, out start_autocalibration_t output);
		[DllImport("ivm", CallingConvention=CallingConvention.Cdecl)]
		public static extern Result ivm_get_status(int id, out get_status_t output);
		[DllImport("ivm", CallingConvention=CallingConvention.Cdecl)]
		public static extern Result ivm_get_measurement_settings(int id, out measurement_settings_t output);
		[DllImport("ivm", CallingConvention=CallingConvention.Cdecl)]
		public static extern Result ivm_set_measurement_settings(int id, ref measurement_settings_t input);
		[DllImport("ivm", CallingConvention=CallingConvention.Cdecl)]
		public static extern Result ivm_start_measurement(int id);
		[DllImport("ivm", CallingConvention=CallingConvention.Cdecl)]
		public static extern Result ivm_check_measurement_status(int id, out check_measurement_status_t output);
		[DllImport("ivm", CallingConvention=CallingConvention.Cdecl)]
		public static extern Result ivm_get_calibration_settings(int id, out calibration_settings_t output);
		[DllImport("ivm", CallingConvention=CallingConvention.Cdecl)]
		public static extern Result ivm_set_calibration_settings(int id, ref calibration_settings_t input);
		[DllImport("ivm", CallingConvention=CallingConvention.Cdecl)]
		public static extern Result ivm_get_measurement_raw(int id, ref in_get_measurement_raw_t input, out out_get_measurement_raw_t output);
		[DllImport("ivm", CallingConvention=CallingConvention.Cdecl)]
		public static extern Result ivm_get_device_rank(int id, out get_device_rank_t output);
	};
};